use crate::nested_type::Type;
use ::anyhow::Result;

use pest::{iterators::Pair, Parser};

#[derive(Parser)]
#[grammar = "grammar/type.pest"]
pub struct TypeParser;

fn parse_type_array(pair: Pair<Rule>) -> Result<Vec<Box<Type>>> {
    let mut inner_rules = pair.into_inner();
    let types = inner_rules.map( |pair| parse_type_object(pair).unwrap_or_default().into()).collect();
    Ok(types)
}

fn parse_type_object(pair: Pair<Rule>) -> Result<Type> {
    // verify that it's a type object ?
    let mut inner_rules = pair.into_inner();
    let repr = inner_rules
        .next()
        .expect("Malformed parsing")
        .as_str();
    let subtypes = match inner_rules.next() {
        Some(a) => parse_type_array(a)?,
        None => vec![]
    };
    Ok(Type {
        repr: repr.into(),
        subtypes: subtypes,
    })
}

fn parse_nested_type(pair: Pair<Rule>) -> Result<Type>{
    match pair.as_rule() {
        Rule::type_object => parse_type_object(pair),
        _ => unreachable!(),
    }
}

pub fn parse(s: String) -> Result<Type> {
    let t = match TypeParser::parse(Rule::nested_type, s.as_str())?.next() {
        Some(v) => v,
        //TODO: return a custom error
        None => panic!("nothing have been parsed"),
    };

    Ok(parse_nested_type(t)?)
}
